#tag Class
Protected Class LDS_DesktopWindowListable
Inherits DesktopWindow
	#tag Event
		Sub Activated()
		  system.DebugLog("Activated "+self.title)
		  mMinimized=false
		  windowActivate(self)
		  raiseevent Activated
		End Sub
	#tag EndEvent

	#tag Event
		Sub Closing()
		  system.DebugLog("Window Closing")
		  windowRemove(self)
		  RaiseEvent Closing
		  
		  LDS_DesktopWindowMenuItem.removeMenuForWindow(self)
		End Sub
	#tag EndEvent

	#tag Event
		Sub Minimized()
		  system.DebugLog("Window Minimized")
		  mMinimized=true
		  windowMinimize(self)
		  RaiseEvent Minimized
		End Sub
	#tag EndEvent

	#tag Event
		Sub Opening()
		  system.DebugLog("Window Opening")
		  mMinimized=false
		  var windowTitle as string = getUniqueWindowTitle(self.title)
		  if(windowTitle <> self.title) then
		    self.title = windowTitle
		  end
		  LDS_DesktopWindowMenuItem.addMenuForWindow(self)
		  
		  RaiseEvent Opening
		  
		  
		End Sub
	#tag EndEvent


	#tag Method, Flags = &h0
		Sub Constructor()
		  // Calling the overridden superclass constructor.
		  windowAdd(self)
		  Super.Constructor
		  
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Shared Function getUniqueWindowTitle(ti as string) As string
		  // this function will (for newly created windows, not those from files being opened)
		  // it will check the window title against all the application windows, if the title is found, it will repeat with increasing suffix appended until a unique title is found
		  var myOriginalTitle as string = ti
		  var myIndex as integer = 0
		  var myTitle as string =myOriginalTitle
		  
		  while(windowDoesExistWithTitle(myTitle))
		    WindowUniqueTitleSuffixCounter = WindowUniqueTitleSuffixCounter+1
		    myIndex = WindowUniqueTitleSuffixCounter
		    myTitle = myOriginalTitle + " "+myIndex.toString
		  wend
		  return myTitle
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function isShowing() As boolean
		  if(self.visible=false or self.mMinimized) then return false
		  return true
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Shared Function topWindow() As LDS_DesktopWindowListable
		  if(WindowList.count>0 and WindowList(0).isShowing) then return WindowList(0)
		  return nil
		End Function
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Shared Sub windowActivate(win as LDS_DesktopWindowListable)
		  var ix as integer
		  windowRemove(win)
		  WindowList.AddAt(0,win)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Shared Sub windowAdd(win as LDS_DesktopWindowListable)
		  WindowList.Add(win)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Shared Function windowDoesExistWithTitle(ti as string) As boolean
		  for each aWin as DesktopWindow in app.Windows
		    if(aWin.Title = ti) then
		      return true
		    end
		  next
		  return false
		End Function
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Shared Sub windowMinimize(win as LDS_DesktopWindowListable)
		  var ix as integer
		  windowRemove(win)
		  ix = WindowList.LastIndex
		  while (ix>=0 and WindowList(ix).mMinimized)
		    ix=ix-1
		  wend
		  ix=ix+1
		  WindowList.AddAt(ix,win)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Shared Sub windowRemove(win as LDS_DesktopWindowListable)
		  var ix as integer
		  for ix = 0 to WindowList.LastIndex
		    if(WindowList(ix) = win) then
		      WindowList.RemoveAt(ix)
		      exit
		    end
		  next
		  
		End Sub
	#tag EndMethod


	#tag Hook, Flags = &h0
		Event Activated()
	#tag EndHook

	#tag Hook, Flags = &h0
		Event Closing()
	#tag EndHook

	#tag Hook, Flags = &h0
		Event Minimized()
	#tag EndHook

	#tag Hook, Flags = &h0
		Event Opening()
	#tag EndHook


	#tag Property, Flags = &h1
		Protected mMinimized As boolean = false
	#tag EndProperty

	#tag Property, Flags = &h0
		Shared WindowList() As LDS_DesktopWindowListable
	#tag EndProperty

	#tag Property, Flags = &h1
		Protected Shared WindowUniqueTitleSuffixCounter As Integer = 0
	#tag EndProperty


	#tag ViewBehavior
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Interfaces"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Width"
			Visible=true
			Group="Size"
			InitialValue="600"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Height"
			Visible=true
			Group="Size"
			InitialValue="400"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MinimumWidth"
			Visible=true
			Group="Size"
			InitialValue="64"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MinimumHeight"
			Visible=true
			Group="Size"
			InitialValue="64"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MaximumWidth"
			Visible=true
			Group="Size"
			InitialValue="32000"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MaximumHeight"
			Visible=true
			Group="Size"
			InitialValue="32000"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Type"
			Visible=true
			Group="Frame"
			InitialValue="0"
			Type="Types"
			EditorType="Enum"
			#tag EnumValues
				"0 - Document"
				"1 - Movable Modal"
				"2 - Modal Dialog"
				"3 - Floating Window"
				"4 - Plain Box"
				"5 - Shadowed Box"
				"6 - Rounded Window"
				"7 - Global Floating Window"
				"8 - Sheet Window"
				"9 - Metal Window"
				"11 - Modeless Dialog"
			#tag EndEnumValues
		#tag EndViewProperty
		#tag ViewProperty
			Name="Title"
			Visible=true
			Group="Frame"
			InitialValue="Untitled"
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="HasCloseButton"
			Visible=true
			Group="Frame"
			InitialValue="True"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="HasMaximizeButton"
			Visible=true
			Group="Frame"
			InitialValue="True"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="HasMinimizeButton"
			Visible=true
			Group="Frame"
			InitialValue="True"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="HasFullScreenButton"
			Visible=true
			Group="Frame"
			InitialValue="False"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Resizeable"
			Visible=true
			Group="Frame"
			InitialValue="True"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Composite"
			Visible=false
			Group="OS X (Carbon)"
			InitialValue="False"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MacProcID"
			Visible=false
			Group="OS X (Carbon)"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="FullScreen"
			Visible=false
			Group="Behavior"
			InitialValue="False"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="DefaultLocation"
			Visible=true
			Group="Behavior"
			InitialValue="2"
			Type="Locations"
			EditorType="Enum"
			#tag EnumValues
				"0 - Default"
				"1 - Parent Window"
				"2 - Main Screen"
				"3 - Parent Window Screen"
				"4 - Stagger"
			#tag EndEnumValues
		#tag EndViewProperty
		#tag ViewProperty
			Name="Visible"
			Visible=true
			Group="Behavior"
			InitialValue="True"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ImplicitInstance"
			Visible=true
			Group="Windows Behavior"
			InitialValue="True"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="HasBackgroundColor"
			Visible=true
			Group="Background"
			InitialValue="False"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="BackgroundColor"
			Visible=true
			Group="Background"
			InitialValue="&cFFFFFF"
			Type="ColorGroup"
			EditorType="ColorGroup"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Backdrop"
			Visible=true
			Group="Background"
			InitialValue=""
			Type="Picture"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MenuBar"
			Visible=true
			Group="Menus"
			InitialValue=""
			Type="DesktopMenuBar"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="MenuBarVisible"
			Visible=true
			Group="Deprecated"
			InitialValue="False"
			Type="Boolean"
			EditorType=""
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
